# Copyright (c) 2002 ETH Zrich, Benno Luthiger. All rights reserved.
# See also LICENSE.txt

# python
import sys, string

# zope
from OFS import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import Globals, zLOG

# Formulator
from Products.Formulator.Form import ZMIForm
from Products.Formulator.Errors import FormValidationError
from Products.Formulator import StandardFields

# Configurable
from helpers import add_and_edit

_ViewManagementScreens = 'View management screens'

class MIConfigurables(SimpleItem.SimpleItem):
    """Management Interface for Configurable Objects.
    """
    
    meta_type = 'Configurables Management Interface'
    
    security = ClassSecurityInfo()
    
    manage_options = (
        {'label':'Edit', 'action':'manage_interfaceForm'},
        ) + SimpleItem.SimpleItem.manage_options

    security.declareProtected(_ViewManagementScreens, 'manage_interfaceForm')
    manage_interfaceForm = PageTemplateFile(
        'www/MIConfigurablesInterface', globals(),  __name__='manage_interfaceForm')

    security.declareProtected(_ViewManagementScreens, 'manage_main')
    manage_main = manage_interfaceForm
    
    def __init__(self, id, title):
        self.id = id
        self.title = title
        self._search_depth = 0
        edit_form = ZMIForm('edit', 'Edit')
        
        depth = StandardFields.IntegerField(
            'search_depth',
            title='Search depth',
            required=1,
            display_width=5)
        depth.values['start'] = 0
        edit_form._setObject(depth.id, depth)
        self.edit_form = edit_form

    security.declareProtected(_ViewManagementScreens, 'edit_form')
    
    security.declareProtected(_ViewManagementScreens, 'search_depth')
    def search_depth(self):
        """Returns the setting for the search depth for Configurables."""
        return self._search_depth
    
    security.declareProtected(_ViewManagementScreens, 'get_configurables')
    def get_configurables(self):
        """Returns a list of Configurables Objects according to the search depth."""
        this_container = self.aq_parent
        out = this_container.objectValues('Configurable Object')
        if self._search_depth > 0:
            out = out + self.get_subfolder_configurables(this_container, 1)
        out.sort(lambda x, y: cmp(x.name(), y.name()))
        return out
    
    def get_subfolder_configurables(self, container, depth):
        out = []
        for object in container.objectValues():
            if object.isPrincipiaFolderish:
                out = out + object.objectValues('Configurable Object')
            if self._search_depth > depth:
                out = out + self.get_subfolder_configurables(object, depth+1)
        return out
    
    security.declareProtected(_ViewManagementScreens, 'manage_edit')
    def manage_edit(self, REQUEST):
        """Manage method to update the value stored by this Configurable Object."""
        configurable = self.get_configurable(self.aq_parent, REQUEST.get('object'))
        configurable.manage_edit(REQUEST, 0)
        return self.manage_main(manage_tabs_message="Changed settings for %s" %configurable.name())
    
    def get_configurable(self, this_root, path_to):
        path = path_to.split('/')
        object = this_root
        for element in path[2:]:
            object = getattr(object, element)
        return object

    security.declareProtected(_ViewManagementScreens, 'manage_depth')
    def manage_depth(self, REQUEST):
        """Manage method to update value for the search depth of Configurables."""
        try:
            result = self.edit_form.validate_all(REQUEST)
        except FormValidationError, e:
            messages = ["Validation error(s)"]
            # loop through all error texts and generate messages for it
            for error in e.errors:
                messages.append("%s: %s" % (error.field.get_value('title'),
                                            error.error_text))
            # join them all together in a big message
            message = string.join(messages, "<br>")
            # return to manage_editForm showing this failure message 
            return self.manage_main(self, REQUEST,
                                    manage_tabs_message=message)
        self._search_depth = result['search_depth']
        return self.manage_main(manage_tabs_message="Changed value for search depth.")

Globals.InitializeClass(MIConfigurables)

manage_addMIConfigurablesForm = PageTemplateFile(
    "www/MIConfigurablesAdd", globals(),
    __name__='manage_addMIConfigurablesForm')

def manage_addMIConfigurables(self, id, title='', REQUEST=None):
    """Add an instance of MIConfigurables."""
    object = MIConfigurables(id, title)
    self._setObject(id, object)
    object = getattr(self, id)
    add_and_edit(self, id, REQUEST)
    return ''
